import React from 'react';
import '../../css/style.css';
import App from "./App";
import * as d3 from "d3";
import {event as currentEvent} from 'd3';

class PrixM2 extends React.Component {

  constructor(props){
    super(props);
    this.state = {
      data: []
    }
  }

  getData(){
    fetch(`https://${document.domain}:8443/prix-m2`)
      .then(response => {
        if (response.ok) {
          return response.json();
        } else {
          throw new Error('error2');
        }
      })
      .then(data => {
        this.setState({data: data });
        this.drawLineChart();
      })
      .catch(error => console.log("error"));
  }

  componentDidMount(){
    this.getData();
  }

  drawLineChart() {
    const data =  this.state.data
    console.log(data)

    const margin = {top: 20, right: 30, bottom: 30, left: 60};
    const width = 800 - margin.left - margin.right;
    const height = 400 - margin.top - margin.bottom;

    const parseTime = d3.timeParse("%Y-%m-%d %H:%M:%S");

    const x = d3.scaleTime()
      .range([0, width]);

    const y = d3.scaleLinear()
      .range([height, 0]);

    console.log(data)
    const line = d3.line()
      .x(d => x(d.month))
      .y(d => y(d.moyenne_surface));

    const svg = d3.select("#chart").html("")
      .append("svg")
      .attr("id", "svg")
      .attr("width", width + margin.left + margin.right)
      .attr("height", height + margin.top + margin.bottom)
      .append("g")
      .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    var map = {};
    data.forEach(function(d) {
      d.month = parseTime(d.month);
      d.moyenne_surface = +d.moyenne_surface;
      map[d.month] = d; //pour mouseSelect plus tard
    });

    x.domain(d3.extent(data, d => d.month ));
    y.domain(d3.extent(data, d => d.moyenne_surface))

    svg.append("g")
      .attr("transform", "translate(0," + height + ")")
      .call(d3.axisBottom(x));

    svg.append("g")
      .call(d3.axisLeft(y))
      .append("text")
      .attr("fill", "#000")
      .attr("transform", "rotate(-90)")
      .attr("y", 6)
      .attr("dy", "0.71em")
      .style("text-anchor", "end")
      .text("Pts");

    svg.selectAll("y axis").data(y.ticks(10)).enter()
      .append("line")
      .attr("class", "horizontalGrid")
      .attr("x1", 0)
      .attr("x2", width)
      .attr("y1", d => y(d))
      .attr("y2", d => y(d));

    svg.append("path")
      .datum(data)
      .attr("class", "line")
      .attr("d", line);

    var div = d3.select("body").append("div")
      .attr("class", "tooltip")
      .attr("x", width - 300)
      .attr("y", 0)
      .style("opacity", 0);

    var verticalLine = svg.append("line")
      .attr("class", "verticalLine")
      .attr("x1",0)
      .attr("y1",0)
      .attr("x2",0)
      .attr("y2",height)
      .style("opacity", 0);

    d3.select("#chart").on("mousemove", function() {
      // Récupération de la position X & Y de la souris.
      var mouse_x = d3.mouse(this)[0];
      var mouse_y = d3.mouse(this)[1];

      // Si la position de la souris est en dehors de la zone du graphique, on arrête le traitement
      if (mouse_x < margin.left || mouse_x > (width + margin.left) || mouse_y < margin.top || mouse_y > (400 - margin.bottom)) {
        return ;
      }

      // Grâce à la fonction 'invert' nous récupérons la date correspondant à notre position
      // A noter, il faut soustraire la marge à gauche pour que la valeur soit correct.
      var selectedDate = x.invert(mouse_x - margin.left);

      // Positionnement de la barre verticale toujours en tenant compte de la marge
      verticalLine.attr("x1", mouse_x - margin.left);
      verticalLine.attr("x2", mouse_x - margin.left);
      verticalLine.style("opacity", 1);

      // Le revert est précis à la milliseconde, ce qui n'est pas le cas de nos données
      selectedDate.setHours(0,0,0,0);
      var entry = map[selectedDate];
      if (typeof entry === "undefined") {
        return ;
      }
      // Si une entrée existe pour la date sélectionnée nous pouvons afficher les données.

      // Le comportement est équivalent aux précédents exemples pour le tooltip.
      div.style("opacity", .9);
      div.style("left", (currentEvent.pageX + 30) + "px")
        .style("top", (currentEvent.pageY - 60) + "px")
        .html("<b>Date : </b>" + dateFormat(entry.month) + "<br>"
          +   "<b>Moy. Surface : </b>" + entry.moyenne_surface + "<br>");
    }).on("mouseout", function() {
      var mouse_x = d3.mouse(this)[0];
      var mouse_y = d3.mouse(this)[1];

      // Si la position de la souris est en dehors de la zone du graphique, on masque la ligne et le tooltip
      if (mouse_x < margin.left || mouse_x > (width + margin.left) || mouse_y < margin.top || mouse_y > (400 - margin.bottom)) {
        div.style("opacity", 0);
        verticalLine.style("opacity", 0);
      }
    });
  }

  render() {
    return  (
      <div>
        <App/>
        <h2>Série temporelle</h2>
        <div id="chart">
          Chargement...
        </div>
      </div>
    )
  }
}

export default PrixM2;
